package java.time.format;

import java.util.Locale;
import java.time.temporal.TemporalQuery;
import java.time.temporal.ValueRange;
import java.time.temporal.TemporalField;
import java.time.chrono.ChronoLocalDate;
import java.time.chrono.IsoChronology;
import java.time.DateTimeException;
import java.time.Instant;
import java.time.ZoneOffset;
import java.time.temporal.ChronoField;
import java.util.Objects;
import java.time.ZoneId;
import java.time.temporal.TemporalQueries;
import java.time.chrono.Chronology;
import java.time.temporal.TemporalAccessor;

final class DateTimePrintContext
{
    private TemporalAccessor temporal;
    private DateTimeFormatter formatter;
    private int optional;
    
    DateTimePrintContext(TemporalAccessor temporalAccessor, DateTimeFormatter formatter) {
        this.temporal = adjust(temporalAccessor, formatter);
        this.formatter = formatter;
    }
    
    private static TemporalAccessor adjust(final TemporalAccessor obj, DateTimeFormatter dateTimeFormatter) {
        Chronology chronology = dateTimeFormatter.getChronology();
        ZoneId zone = dateTimeFormatter.getZone();
        if (chronology == null && zone == null) {
            return obj;
        }
        final Chronology b = (Chronology)obj.query(TemporalQueries.chronology());
        final ZoneId b2 = (ZoneId)obj.query(TemporalQueries.zoneId());
        if (Objects.equals(chronology, b)) {
            chronology = null;
        }
        if (Objects.equals(zone, b2)) {
            zone = null;
        }
        if (chronology == null && zone == null) {
            return obj;
        }
        b = ((chronology != null) ? chronology : b);
        if (zone != null) {
            if (obj.isSupported(ChronoField.INSTANT_SECONDS)) {
                return ((b != null) ? b : IsoChronology.INSTANCE).zonedDateTime(Instant.from(obj), zone);
            }
            if (zone.normalized() instanceof ZoneOffset && obj.isSupported(ChronoField.OFFSET_SECONDS) && obj.get(ChronoField.OFFSET_SECONDS) != zone.getRules().getOffset(Instant.EPOCH).getTotalSeconds()) {
                throw new DateTimeException("Unable to apply override zone '" + zone + "' because the temporal object being formatted has a different offset but does not represent an instant: " + obj);
            }
        }
        b2 = ((zone != null) ? zone : b2);
        final ChronoLocalDate date;
        if (chronology != null) {
            if (obj.isSupported(ChronoField.EPOCH_DAY)) {
                date = b.date(obj);
            }
            else {
                if (chronology != IsoChronology.INSTANCE || b != null) {
                    ChronoField[] values = ChronoField.values();
                    int length = values.length;
                    int i = 0;
                    while (i < length) {
                        ChronoField chronoField = values[i];
                        if (chronoField.isDateBased() && obj.isSupported(chronoField)) {
                            throw new DateTimeException("Unable to apply override chronology '" + chronology + "' because the temporal object being formatted contains date fields but does not represent a whole date: " + obj);
                        }
                        ++i;
                    }
                }
                date = null;
            }
        }
        else {
            date = null;
        }
        return new TemporalAccessor() {
            @Override
            public boolean isSupported(TemporalField field) {
                if (date != null && field.isDateBased()) {
                    return date.isSupported(field);
                }
                return obj.isSupported(field);
            }
            
            @Override
            public ValueRange range(TemporalField temporalField) {
                if (date != null && temporalField.isDateBased()) {
                    return date.range(temporalField);
                }
                return obj.range(temporalField);
            }
            
            @Override
            public long getLong(TemporalField temporalField) {
                if (date != null && temporalField.isDateBased()) {
                    return date.getLong(temporalField);
                }
                return obj.getLong(temporalField);
            }
            
            @Override
            public <R> R query(TemporalQuery<R> query) {
                if (query == TemporalQueries.chronology()) {
                    return b;
                }
                if (query == TemporalQueries.zoneId()) {
                    return b2;
                }
                if (query == TemporalQueries.precision()) {
                    return obj.query(query);
                }
                return query.queryFrom(this);
            }
        };
    }
    
    TemporalAccessor getTemporal() {
        return this.temporal;
    }
    
    Locale getLocale() {
        return this.formatter.getLocale();
    }
    
    DecimalStyle getDecimalStyle() {
        return this.formatter.getDecimalStyle();
    }
    
    void startOptional() {
        ++this.optional;
    }
    
    void endOptional() {
        --this.optional;
    }
    
     <R> R getValue(TemporalQuery<R> query) {
        Object query2 = this.temporal.query(query);
        if (query2 == null && this.optional == 0) {
            throw new DateTimeException("Unable to extract value: " + this.temporal.getClass());
        }
        return query2;
    }
    
    Long getValue(TemporalField temporalField) {
        try {
            return Long.valueOf(this.temporal.getLong(temporalField));
        }
        catch (DateTimeException ex) {
            if (this.optional > 0) {
                return null;
            }
            throw ex;
        }
    }
    
    @Override
    public String toString() {
        return this.temporal.toString();
    }
}
